/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.core.util;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;

import org.ow2.opensuit.core.impl.UrlWatcher;


/**
 * Helper interface for building Urls with parameters, and managing url encoding.
 */
public class UrlBuilder
{
	protected String _requestUri;
	protected ArrayList<String> _names;
	protected ArrayList<String> _values;
	
	/**
	 * Creates a new UrlBuilder with given request uri.
	 * @param iRequestURI The url request uri.
	 */
	public UrlBuilder(String iRequestURI)
	{
		_requestUri = iRequestURI;
	}
	/**
	 * Returns the url request uri.
	 * @return
	 */
	public String getRequestUri()
	{
		return _requestUri;
	}
	/**
	 * Sets the given Url parameter.
	 * @param iName Parameter name.
	 * @param iValue Parameter value.
	 * @return Returns this object, to allows chaining calls.
	 */
	public UrlBuilder setParameter(String iName, String iValue)
	{
		if(iName != null)
		{
			if(_names == null)
			{
				_names = new ArrayList<String>(1);
				_values = new ArrayList<String>(1);
			}
			if(iValue == null)
				iValue = "";
			int i = _names == null ? -1 : _names.indexOf(iName);
			if(i >= 0)
			{
				// --- change value
				_values.set(i, iValue);
			}
			else
			{
				// --- add value
				_names.add(iName);
				_values.add(iValue);
			}
		}
		return this;
	}
	/**
	 * Removes the parameter with given name.
	 * @param iName The parameter name to remove.
	 * @return this object (to chain calls).
	 */
	public UrlBuilder removeParameter(String iName)
	{
		int i = _names == null ? -1 : _names.indexOf(iName);
		if(i >= 0)
		{
			_names.remove(i);
			_values.remove(i);
		}
		return this;
	}
	/**
	 * Returns the list of parameters in the Url.
	 * @return
	 */
	public List<String> getParameterNames()
	{
		return _names;
	}
	/**
	 * Returns the url parameter with given name.
	 * @param iName The parameter name.
	 * @return The parameter value.
	 */
	public String getParameterValue(String iName)
	{
		int i = _names == null ? -1 : _names.indexOf(iName);
		if(i >= 0)
			return _values.get(i);
		return  null;
	}

	/**
	 * Serializes the Url to its string form.
	 * @param iCharSet Gives the charset to use to encode the url.
	 * @param iEncodeHtml Tells whether the url has to be HTML encoded (in addition to url encoding).
	 * @return The encoded url with all its parameters.
	 */
	public String toUrl(String iCharSet, boolean iEncodeHtml)
	{
		StringBuffer sb = new StringBuffer(getRequestUri());
		if(_names != null)
		{
			for(int i=0; i<_names.size(); i++)
			{
				String name = _names.get(i);
				String value = _values.get(i);
				if(i == 0)
					sb.append('?');
				else
					sb.append(iEncodeHtml ? "&amp;" : "&");
					
				sb.append(name);
				sb.append('=');
				if(iCharSet != null)
				{
					try
	                {
						value = URLEncoder.encode(value, iCharSet);
	                }
	                catch(UnsupportedEncodingException e)
	                {
		                e.printStackTrace();
	                }
				}
                sb.append(value);
			}
		}
		if(UrlWatcher.getInstalled() != null)
			UrlWatcher.getInstalled().addUrl(sb.toString());
		return sb.toString();
	}
	@Override
	public String toString()
	{
		return toUrl(null, false);
	}
}
