/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.binding;

import java.lang.reflect.Type;

import javax.servlet.http.HttpServletRequest;

import org.ow2.opensuit.cel.ICompilationMessage;
import org.ow2.opensuit.cel.ICompilationResult;
import org.ow2.opensuit.cel.IExpression;
import org.ow2.opensuit.core.expression.ExpressionUtils;
import org.ow2.opensuit.core.util.HtmlUtils;
import org.ow2.opensuit.xml.base.Application;
import org.ow2.opensuit.xml.interfaces.IBeanProvider;
import org.ow2.opensuit.xmlmap.annotations.XmlAncestor;
import org.ow2.opensuit.xmlmap.annotations.XmlContent;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlSimpleType;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("This simple type allows to declare expressions that interact with the presentation layer:" +
		"<ul>" +
		"<li>get values," +
		"<li>set values (when submitting a form for instance)," +
		"<li>call methods (to react to application events)." +
		"</ul>" +
		"The underlying engine is fully compatible with Java Unified Expression Language (see http://java.sun.com/products/jsp/reference/techart/unifiedEL.html)," +
		"except that expressions are <strong>compiled</strong>, and <strong>strongly-typed</strong> (no weak-type conversion).")
@XmlSimpleType(base=String.class)
public class Expression implements IInitializable
{
	@XmlAncestor
	private Application root;
	
	@XmlAncestor
	private IBeanProvider parentProvider;

	@XmlDoc("Expression.")
	@XmlContent
	private String expressionString;
	
	private String _id;
	private IExpression _expression;

	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
//		super.initialize(initSupport, additionalSupport);
		
		expressionString = expressionString.trim();
		ICompilationResult compResult = ExpressionUtils.compileExpression(expressionString, root, parentProvider);
		if(compResult.getMessages() != null)
		{
			for(ICompilationMessage message : compResult.getMessages())
			{
				int level = IInitializationSupport.ERROR;
				if(message.getLevel() == ICompilationMessage.WARNING_LEVEL)
					level = IInitializationSupport.WARNING;
				else if(message.getLevel() == ICompilationMessage.INFO_LEVEL)
					level = IInitializationSupport.INFO;
				
//				initSupport.addValidationMessage(this, "content", level, message.getMessage()+"\r\n..."+expressionString+"\r\n"+makeCharArrow(message.getPosition()-2));
				initSupport.addValidationMessage(this, "content", level, message.getMessage()+": \r\n"+(expressionString.substring(0, message.getPosition())));
			}
		}
		_expression = compResult.getExpression();
		
		// --- check type
		// TODO
	}
	private static String makeCharArrow(int position)
	{
		StringBuffer sb = new StringBuffer();
		for(int i=0; i<position-1; i++)
			sb.append('-');
		sb.append('^');
		return sb.toString();
	}
	public Type getGenericType()
	{
		if(_expression == null)
			return null;
	    return _expression.getGenericType();
	}
	public Class<?> getType()
	{
		if(_expression == null)
			return null;
	    return _expression.getType();
	}
	public boolean isConvertible(Class<?> type)
	{
		if(_expression == null)
			return false;
		return ExpressionUtils.getTypeConverter().isConvertible(_expression.getType(), type);
	}

	public boolean hasErrors()
	{
	    return _expression == null;
	}
	public Object invoke(HttpServletRequest iRequest) throws Exception
	{
		if(_expression == null)
			return null;
		return ExpressionUtils.invokeExpression(_expression, root, parentProvider, iRequest);
	}
	public <T> T invoke(HttpServletRequest iRequest, Class<T> type) throws Exception
	{
		if(_expression == null)
			return null;
		Object ret = ExpressionUtils.invokeExpression(_expression, root, parentProvider, iRequest);
		return ExpressionUtils.getTypeConverter().convert(ret, type);
	}
	public String getID()
	{
		if(_id == null)
			_id = HtmlUtils.formatId(expressionString);
	    return _id;
	}
	public boolean isStaticValue()
	{
		if(_expression == null)
			return true;
	    return _expression.isStaticValue();
	}
	public boolean isReadOnly()
	{
		if(_expression == null)
			return true;
	    return _expression.isReadOnly();
	}
	public void set(HttpServletRequest iRequest, Object iValue) throws Exception
	{
		if(_expression == null)
			return;
		ExpressionUtils.setExpression(_expression, root, parentProvider, iRequest, iValue);
	}
	@Override
	public String toString()
	{
		return expressionString;
	}

}
