/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.page;

import java.io.PrintWriter;
import java.lang.reflect.Type;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.core.expression.ExpressionUtils;
import org.ow2.opensuit.core.session.OpenSuitSession;
import org.ow2.opensuit.core.util.HtmlUtils;
import org.ow2.opensuit.xml.base.Application;
import org.ow2.opensuit.xml.base.binding.Do;
import org.ow2.opensuit.xml.base.binding.Expression;
import org.ow2.opensuit.xml.base.html.CssStyleSheet;
import org.ow2.opensuit.xml.base.html.IFrame;
import org.ow2.opensuit.xml.base.page.ctx.IPageContextDef;
import org.ow2.opensuit.xml.interfaces.BaseBeanProvider;
import org.ow2.opensuit.xmlmap.annotations.XmlAncestor;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlChildren;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;
import org.ow2.opensuit.xmlmap.interfaces.IInstantiationContext;



@XmlDoc("The base page component.")
@XmlElement
public abstract class BasePage extends BaseBeanProvider implements IPage, IInitializable
{
	@XmlAncestor
	private Application application;
	
	@XmlDoc("A non-null value triggers the automatic page reload (in seconds).")
	@XmlAttribute(required=false, name="RefreshDelay")
	private int refresh = 0;
	
	@XmlDoc("The name of the frame this page displays in.<br>" +
			"If undefined, the default frame will be used.")
	@XmlAttribute(name="FrameName", required=false)
	private String frameName;
	
//	@XmlDoc("The page's name. Must be unique in the application. ")
//	@XmlAttribute(name="Name")
//	private String name;
	private String path;
	
	@XmlDoc("The page context definition.")
	@XmlChild(name="Context")
	private IPageContextDef context;
	
	@XmlDoc("An action that is triggered right before the page is rendered.")
	@XmlChildren(name="OnPreRender", minOccurs=0)
	private Do[] onPreRender;
	
	@XmlDoc("The page displayed title.")
	@XmlChild(name="Title")
	private Expression title;
	
	// --- [+] IIdentifiable implementation
	public String getPathID()
	{
		return "/page"+path;
	}
	// --- [-] IIdentifiable implementation

	public void initialize(IInitializationSupport initSupport, IInstantiationContext instContext)
	{
		super.initialize(initSupport, instContext);
		
		if(application.getFrame(frameName) == null)
			initSupport.addValidationMessage(this, "FrameName", IInitializationSupport.ERROR, "Frame '"+frameName+"' not found.");
		
		// --- trigger context initialization
		initSupport.initialize(context);
	}
	public IPageContextDef getContextDefinition()
	{
	    return context;
	}
	public void setPath(String path)
	{
		this.path = path;
	}
	public String getPath()
	{
	    return path;
	}
	public String getFrameName()
	{
	    return frameName;
	}
	public String getTitle(HttpServletRequest iRequest) throws Exception
	{
		return ExpressionUtils.getMessage(title, iRequest);
	}
	public void preRender(HttpServletRequest iRequest) throws Exception
	{
		if(onPreRender != null)
		{
			for(int i=0; i<onPreRender.length; i++)
				onPreRender[i].invoke(iRequest);
		}
	}
	public void servePage(HttpServletRequest iRequest, HttpServletResponse iResponse) throws Exception
	{
		OpenSuitSession session = OpenSuitSession.getSession(iRequest);
		iRequest.setAttribute("_RENDER_PAGE_", this);
		
		// --- 1: pre-render
//		preRender(iRequest);
		application.getFrame(getFrameName()).preRender(iRequest);
		
		// --- 2: render
		
		/*
		 * W3C rules:
		 * 
		 * XHTML 1.0 strict:
		 * <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
		 * <html xmlns="http://www.w3.org/1999/xhtml" lang="en" xml:lang="en">
		 * 
		 * XHTML 1.0 transitional:
		 * <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
		 * <html xmlns="http://www.w3.org/1999/xhtml" lang="en" xml:lang="en">
		 * 
		 * HTML 4.0.1 strict:
		 * <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">
		 * 
		 * HTML 4.0.1 transitional:
		 * <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
		 * <html lang="en">
		 * 
		 * XHTML 1.1:
		 * <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">
		 * <html lang="en">
		 * 
		 * 
		 * With XHTML, the content type must be "application/xhtml+xml" or "text/xml" (for IE)
		 */
		// --- render response (html)
		iResponse.setContentType("text/html;charset="+session.getLocaleConfig().getCharSet());
//		iResponse.setHeader("Cache-Control", "no-store"); //HTTP 1.1
		iResponse.setHeader("Cache-Control", "no-cache"); //HTTP 1.1
		iResponse.setHeader("Pragma", "no-cache"); //HTTP 1.0
		iResponse.setDateHeader("Expires", 0); //prevents caching at the proxy server

		PrintWriter htmlWriter = iResponse.getWriter();
		
		// --- 3: render
		//TODO: other headers
		
		htmlWriter.println(HtmlUtils.getHTML401TransitionalDoctype());
		htmlWriter.print("<html lang=\"");
		htmlWriter.print(session.getLocaleConfig().getLocale().getLanguage());
		htmlWriter.println("\">");
		htmlWriter.println("<head>");

		// --- write page title
		htmlWriter.print("<title>");
		htmlWriter.print(HtmlUtils.encode2HTML(this.getTitle(iRequest)));
		htmlWriter.println("</title>");

		// --- meta tags
		// html content type
		htmlWriter.println("<meta http-equiv='content-type' content='text/html;charset="+session.getLocaleConfig().getCharSet()+"'>");
		// no cache (for all browsers)
		htmlWriter.println("<meta http-equiv='cache-control' content='no-cache'>");
		htmlWriter.println("<meta http-equiv='pragma' content='no-cache'>");
		htmlWriter.println("<meta http-equiv='expires' content='0'>");
		if(refresh > 0)
			htmlWriter.println("<meta http-equiv='refresh' content='"+refresh+"'>");
		/*
		// --- les liens relatifs le sont par rapport  l'application
		htmlWriter.print("<base href='");
		htmlWriter.print(iRequest.getScheme());
		htmlWriter.print("://");
		htmlWriter.print(iRequest.getServerName());
		htmlWriter.print(":");
		htmlWriter.print(String.valueOf(iRequest.getServerPort()));
		htmlWriter.print(iRequest.getContextPath());
		htmlWriter.print("/");
		htmlWriter.println("'>");
		*/

		// --- import stylesheets
		IFrame frame = application.getFrame(getFrameName());
		CssStyleSheet[] styleSheets = frame.getStyleSheets();
		if(styleSheets != null)
		{
			for(int i=0; i<styleSheets.length; i++)
			{
				htmlWriter.print("<link rel='stylesheet' type='text/css' href='");
				htmlWriter.print(styleSheets[i].getHref(iRequest));
				htmlWriter.println("'>");
			}
		}
		
		htmlWriter.println("</head>");
		
		htmlWriter.println("<body>");
		
		// --- render the frame
		frame.render(iRequest, iResponse);
		
		htmlWriter.println("</body>");
		htmlWriter.println("</html>");
		
		htmlWriter.flush();
		htmlWriter.close();
	}
	// ====================================================================
	// === Bean Provider methods
	// ====================================================================
	public Type getBeanGenericType(String iName) throws UnresolvedBeanError
	{
		Type t = context.getManager().getBeanGenericType(iName);
		if(t != null)
			return t;
		return getParentBeanGenericType(iName);
	}
	public Class<?> getBeanType(String iName) throws UnresolvedBeanError
	{
		Class<?> c = context.getManager().getBeanType(iName);
		if(c != null)
			return c;
		return getParentBeanType(iName);
	}
	public Object getBeanValue(HttpServletRequest iRequest, String iName) throws Exception
	{
		Object o = context.getManager().getBeanValue(iRequest, iName);
		if(o != null)
			return o;
		return getParentBeanValue(iRequest, iName);
	}
}
